@@ -24,13 +24,20 @@ except _metadata.PackageNotFoundError:  # pragma: no cover - fallback for tests
 from .agent import (
     Agent,
     run_interactive,
-    set_system_message_builder,
 )
+from .system_message import set_system_message_builder
+from .session import Session, CliSession
 from .models import Model, OpenAIModel, set_custom_model  # noqa: E402,F401
 from .errors import PygentError, APIError  # noqa: E402,F401
 from .tools import register_tool, tool, clear_tools, reset_tools, remove_tool  # noqa: E402,F401
 from .task_manager import TaskManager  # noqa: E402,F401
 from .task_tools import register_task_tools  # noqa: E402,F401
+from .prompt_library import PROMPT_BUILDERS  # noqa: E402,F401
+from .agent_presets import AGENT_PRESETS, AgentPreset  # noqa: E402,F401
+try:  # optional dependency
+    from .fastapi_app import create_app  # noqa: E402,F401
+except Exception:  # pragma: no cover - optional
+    create_app = None  # type: ignore
 
 __all__ = [
     "Agent",
@@ -41,6 +48,8 @@ __all__ = [
     "OpenAIModel",
     "set_custom_model",
     "set_system_message_builder",
+    "Session",
+    "CliSession",
     "PygentError",
     "APIError",
     "register_tool",
@@ -50,4 +59,8 @@ __all__ = [
     "remove_tool",
     "TaskManager",
     "register_task_tools",
+    "PROMPT_BUILDERS",
+    "AGENT_PRESETS",
+    "AgentPreset",
+    "create_app",
 ]
