@@ -42,9 +42,12 @@ class Command:
 
 def cmd_cmd(agent: Agent, arg: str) -> None:
     """Run a raw shell command in the sandbox."""
-    output = agent.runtime.bash(arg)
     console = Console()
-    console.print(output)
+
+    def _stream(line: str) -> None:
+        console.print(line, end="")
+
+    agent.runtime.bash(arg, stream=_stream)
 
 
 def cmd_cp(agent: Agent, arg: str) -> None:
@@ -184,6 +187,20 @@ def cmd_banned(agent: Agent, arg: str) -> None:
             print(f"{name} not banned")
 
 
+def cmd_confirm_bash(agent: Agent, arg: str) -> None:
+    """Show or toggle confirmation for bash commands."""
+    arg = arg.strip().lower()
+    if not arg:
+        status = "on" if agent.confirm_bash else "off"
+        print(status)
+        return
+    if arg not in {"on", "off"}:
+        print("usage: /confirm-bash [on|off]")
+        return
+    agent.confirm_bash = arg == "on"
+    print("confirmation " + ("enabled" if agent.confirm_bash else "disabled"))
+
+
 def register_command(
     name: str,
     handler: Callable[[Agent, str], Optional[Agent]],
@@ -204,4 +221,5 @@ COMMANDS: Dict[str, Command] = {
     "/save": Command(cmd_save, description="Save workspace and environment to DIR for later use.", usage="/save DIR"),
     "/tools": Command(cmd_tools, description="Enable/disable tools at runtime or list them.", usage="/tools [list|enable NAME|disable NAME]"),
     "/banned": Command(cmd_banned, description="List or modify banned commands.", usage="/banned [list|add CMD|remove CMD]"),
+    "/confirm-bash": Command(cmd_confirm_bash, description="Toggle confirmation before running bash commands.", usage="/confirm-bash [on|off]"),
 }
