@@ -29,3 +29,19 @@ def test_run_py_config_sets_env(tmp_path, monkeypatch):
     run_py_config(cfg)
     assert os.getenv('TEST_VAR') == 'ok'
     monkeypatch.delenv('TEST_VAR', raising=False)
+
+
+def test_run_py_config_sets_system_builder(tmp_path):
+    cfg = tmp_path / 'config.py'
+    cfg.write_text(
+        'from pygent.agent import set_system_message_builder\n'
+        'def b(p, disabled_tools=None):\n'
+        '    return "CUSTOM"\n'
+        'set_system_message_builder(b)\n'
+    )
+    run_py_config(cfg)
+    from pygent.agent import Agent, set_system_message_builder
+
+    ag = Agent()
+    assert ag.system_msg == "CUSTOM"
+    set_system_message_builder(None)
