@@ -75,6 +75,29 @@ def test_register_and_execute_custom_tool():
     assert tools.execute_tool(call, DummyRuntime()) == 'hi bob'
 
 
+def test_execute_tool_handles_exception():
+    def boom(rt):
+        raise RuntimeError('fail')
+
+    register_tool(
+        "boom",
+        "fail",
+        {"type": "object", "properties": {}},
+        lambda rt: boom(rt),
+    )
+
+    call = type('Call', (), {
+        'function': type('Func', (), {
+            'name': 'boom',
+            'arguments': '{}'
+        })
+    })()
+
+    out = tools.execute_tool(call, DummyRuntime())
+    assert 'error' in out.lower()
+    remove_tool('boom')
+
+
 def test_clear_and_reset_tools_updates_prompt():
     clear_tools()
     ag = Agent()
@@ -106,3 +129,40 @@ def test_read_image_data_url(tmp_path):
     assert result.startswith("data:image/png;base64,")
 
 
+def test_read_image_detects_mime_from_contents(tmp_path):
+    rt = Runtime(use_docker=False, workspace=tmp_path)
+    img = tmp_path / "img"
+    img.write_bytes(b"\x89PNG\r\n")
+    result = tools._read_image(rt, path="img")
+    assert result.startswith("data:image/png;base64,")
+
+
+def test_workflow_block_respects_ask_user():
+    reset_tools()
+    ag = Agent()
+    assert "ask_user" in ag.system_msg
+    assert "procceed" in ag.system_msg
+
+    remove_tool("ask_user")
+    ag2 = Agent()
+    assert "ask_user" not in ag2.system_msg
+    assert "procceed" not in ag2.system_msg
+    reset_tools()
+
+
+def test_workflow_block_respects_stop_tool():
+    reset_tools()
+    ag = Agent()
+    assert "stop" in ag.system_msg
+
+    remove_tool("stop")
+    ag2 = Agent()
+    assert "stop" not in ag2.system_msg
+    reset_tools()
+
+
+def test_workflow_block_includes_review_instruction():
+    ag = Agent()
+    assert "verify and test" in ag.system_msg
+
+
